@@ -66,15 +66,20 @@ a symbol, it should be unquoted.."
   nil)
 
 (cl-defmethod magit-section-ident-value ((section ement-room-list-section))
+  "Return ident value for `ement-room-list-section' SECTION.
+Used for caching section visibility."
   ;; FIXME: The name of each taxy could be ambiguous.  Best would be to use the
   ;; hierarchical path, but since the taxys aren't doubly linked, that isn't easily done.
   ;; Could probably be worked around by binding a special variable around the creation of
   ;; the taxy hierarchy that would allow the path to be saved into each taxy.
   (pcase-exhaustive (oref section value)
-    ((and (cl-type taxy-magit-section) it)
+    ;; FIXME(emacs-28): Use `(cl-type taxy-magit-section)' and `(cl-type ement-room)', et
+    ;; al. when requiring Emacs 28.  See
+    ;; <https://github.com/alphapapa/ement.el/issues/272>.
+    ((and (pred taxy-magit-section-p) it)
      (taxy-name it))
-    (`[,(and (cl-type ement-room) room)
-       ,(and (cl-type ement-session) session)]
+    (`[,(and (pred ement-room-p) room)
+       ,(and (pred ement-session-p) session)]
      (vector (ement-user-id (ement-session-user session))
              (ement-room-id room)))
     ((pred null) nil)))
@@ -118,7 +123,7 @@ Set automatically when `ement-room-list-mode' is activated.")
 ;;;;; Persistent variables
 
 (persist-defvar ement-room-list-visibility-cache nil
-                "Applied to `magit-section-visibility-cache', which see.")
+  "Applied to `magit-section-visibility-cache', which see.")
 
 ;;;; Customization
 
@@ -141,16 +146,23 @@ Set automatically when `ement-room-list-mode' is activated.")
 ;;;;; Faces
 
 (defface ement-room-list-direct
-  ;; In case `font-lock-constant-face' is bold, we set the weight to normal, so it can be
-  ;; made bold for unread rooms only.
-  '((t (:weight normal :inherit (font-lock-constant-face ement-room-list-name))))
+  ;; We want to use `font-lock-constant-face' as the base face (because it seems to look
+  ;; nice with most themes), but that face sometimes is defined as bold, which interferes
+  ;; with our ability to use boldness to indicate unread rooms.  But if we override the
+  ;; weight to be normal, even the "People" heading in the room list will not be bold,
+  ;; which group headings should be.  So we make a copy of the face, unset its weight, and
+  ;; inherit from that.
+  (progn
+    (copy-face 'font-lock-constant-face 'ement--font-lock-constant-face)
+    (set-face-attribute 'ement--font-lock-constant-face nil :weight 'unspecified)
+    '((t (:inherit (ement--font-lock-constant-face ement-room-list-name)))))
   "Direct rooms.")
 
 (defface ement-room-list-favourite '((t (:inherit (font-lock-doc-face ement-room-list-name))))
   "Favourite rooms.")
 
 (defface ement-room-list-invited
-  '((t (:inherit italic ement-room-list-name)))
+  '((t (:inherit (italic ement-room-list-name))))
   "Invited rooms.")
 
 (defface ement-room-list-left
@@ -161,7 +173,7 @@ Set automatically when `ement-room-list-mode' is activated.")
   "Low-priority rooms.")
 
 (defface ement-room-list-name
-  '((t (:inherit font-lock-function-name-face button)))
+  '((t (:inherit (font-lock-function-name-face button))))
   "Non-direct rooms.")
 
 (defface ement-room-list-space '((t (:inherit (font-lock-regexp-grouping-backslash ement-room-list-name))))
@@ -169,7 +181,7 @@ Set automatically when `ement-room-list-mode' is activated.")
   :group 'ement-room-list)
 
 (defface ement-room-list-unread
-  '((t (:inherit bold ement-room-list-name)))
+  '((t (:inherit (bold ement-room-list-name))))
   "Unread rooms.")
 
 (defface ement-room-list-recent '((t (:inherit font-lock-warning-face)))
@@ -343,16 +355,17 @@ from recent to non-recent for rooms updated in the past hour.")
     ((membership :status 'leave))
     ;; Group all favorite rooms, which are already sorted first.
     (favourite)
-    ;; Group all low-priority rooms, which are already sorted last, and within that group,
-    ;; group them by their space, if any.
-    (low-priority space)
     ;; Group other rooms which are opened in a buffer.
     (buffer)
     ;; Group other rooms which are unread.
     (unread)
-    ;; Group other rooms which are in a space by freshness, then by space.
+    ;; Group all low-priority rooms, which are already sorted last, and within that group,
+    ;; group them by their space, if any.
+    (low-priority space)
+    ;; Group other non-direct rooms which are in a space by freshness, then by space.
     ((and :name "Spaced"
           :keys ((not space-p)
+                 (not people)
                  space))
      freshness space)
     ;; Group spaces themselves by their parent space (since space headers can't also be
@@ -364,8 +377,8 @@ from recent to non-recent for rooms updated in the past hour.")
           :keys ((not space)
                  (not people)))
      freshness)
-    ;; Group direct rooms by freshness.
-    (people freshness))
+    ;; Group direct rooms by freshness and space.
+    (people freshness space))
   "Default keys."
   :type 'sexp)
 
